"""Combine the pages artifacts from all branches into a single deployment."""

import os
import shutil
from pathlib import Path
import subprocess
from zipfile import ZipFile

import gitlab
import gitlab.exceptions


def fetch_branch_names():
    git_remote_heads = subprocess.run(
        ["git", "ls-remote", "--heads", "-q"],
        capture_output=True,
        text=True,
    )
    return [
        line.split("\t")[-1].split("/")[-1]
        for line in git_remote_heads.stdout.strip().split("\n")
    ]


def fetch_and_combine_artifacts(branch_names):
    SERVER_URL = os.environ["CI_SERVER_URL"]
    PRIVATE_TOKEN = os.environ["GITLAB_PRIVATE_TOKEN"]
    PROJECT_ID = os.environ["CI_PROJECT_ID"]
    DEFAULT_BRANCH = os.environ["CI_DEFAULT_BRANCH"]

    gl = gitlab.Gitlab(SERVER_URL, private_token=PRIVATE_TOKEN)

    # Use lazy=True to avoid fetching from the Projects API
    project = gl.projects.get(PROJECT_ID, lazy=True)

    download_dir = Path.cwd() / "artifacts"
    download_dir.mkdir(exist_ok=True)

    pages_path = Path.cwd() / "public"
    pages_path.mkdir(exist_ok=True)

    for branch_name in branch_names:
        archive_path = download_dir / f"{branch_name}.zip"
        extract_path = download_dir / branch_name

        # Default branch is served from the root
        branch_pages_path = pages_path / branch_name
        if branch_name == DEFAULT_BRANCH:
            branch_pages_path = pages_path

        try:
            # The CI job already fetches the artifacts for the current branch
            if not extract_path.exists():
                # Otherwise download the artifacts
                with archive_path.open("wb") as f:
                    f.write(
                        project.artifacts.download(
                            ref_name=branch_name, job="store_pages"
                        )
                    )

                extract_path.mkdir(exist_ok=True)
                with ZipFile(archive_path, "r") as archive:
                    archive.extractall(extract_path)

                print(f"Artifacts downloaded for branch {branch_name}")

            # Copy the contents of the public folder to the branch specific pages folder
            extract_public_path = extract_path / "public"
            branch_pages_path.mkdir(exist_ok=True)
            for item in extract_public_path.iterdir():
                shutil.move(item, branch_pages_path)

            print(f"Artifacts extracted for branch {branch_name}")

        except gitlab.exceptions.GitlabGetError as e:
            # No artifacts found for this branch
            print(f"No artifacts found for branch {branch_name}: {e}")
            pass
        finally:
            if archive_path.exists():
                archive_path.unlink()


if __name__ == "__main__":
    branch_names = fetch_branch_names()
    fetch_and_combine_artifacts(branch_names)
